//
// EZ-FPGA chamereon USB Logiana board , PROM Emulator Design
//
// Copyright(C) 2003 Tatusuyuki Satoh
//
// Memory Area Map
// +---------+---------------------------------------------+
// |JA  area |Contents                                     |
// +---------+---------------------------------------------+
// |00000000-|PBSRAM Emulation memory 512KB / 128KB        |
// |         | JA[1.0]==2'b00:latch to RD[ 7: 0]           |
// |         | JA[1.0]==2'b01:latch to RD[15: 8]           |
// |         | JA[1.0]==2'b10:latch to RD[24:23]           |
// |-0003FFFF| JA[1.0]==2'b00:latch to RD[31:24] and write |
// +---------+---------------------------------------------+
// |40000000-|Controll Register                            |
// |-4000001F|                                             |
// +---------+---------------------------------------------+
//
// Controll Register (write)
// +--------+---+---+---+---+---+---+---+---+----------------------------+
// |JA      |b7 |b6 |b5 |b4 |b3 |b2 |b1 |b0 | Contents                   |
// +--------+---+---+---+---+---+---+---+---+----------------------------+
// |40000001|A7 |A6 |A5 |A4 |A3 |A2 |A1 |A0 | Trigger Address L          |
// |40000002|A15|A14|A13|A12|A11|A10|A9 |A8 | Trigger Address M          |
// |40000003| X | X | X | X | X |A18|A17|A16| Trigger Address H          |
// |40000006|EMU|RSE|RSL|TPM| X | X | X | X | ModeControll               |
// |40000007| X |FL |R18|R17|R16|R15|R14|R13| address enable mask        |
// +----+---+---+---+---+---+---+---+---+--------------------------------+
//
// Controll Register (read)
// +--------+---+---+---+---+---+---+---+---+----------------------------+
// |JA      |b7 |b6 |b5 |b4 |b3 |b2 |b1 |b0 | Contents                   |
// +--------+---+---+---+---+---+---+---+---+----------------------------+
// |40000000|TD7|TD6|TD5|TD4|TD3|TD2|TD1|TD0| Trace Data                 |
// |40000004| 0 | 0 | 0 | 0 |DLL|PLL|VCC|TRG| Sense VCC & Trigger        |
// |4000000D| 0 | 1 | 0 | 1 | 0 | 0 | 1 | 0 | Fixed Code :'R'(8'h52)     |
// |4000000E| 0 | 1 | 0 | 0 | 0 | 1 | 0 | 1 | Fixed Code :'E'(8'h45)     |
// |4000000F|MV3|MV2|MV1|MV0|RC3|RC2|RC1|RC0| Version Code               |
// +--------+---+---+---+---+---+---+---+---+----------------------------+
//
// Writre Register Spec.
// +-------+-----------------------------------------------+
// |Param. | Spec.                                         |
// +-------+-----------------------------------------------+
// |A      | Triger Ditect Address.                        |
// |[18..0]|                                               |
// +-------+-----------------------------------------------+
// |EMU    | 0=emulate disable, can access by host.        |
// |       | 1=emulate enable , can't access by host.      |
// +-------+-----------------------------------------------+
// |RSE    | Reset Signal                                  |
// |RSL    | RSE=0 : Reset = Hi-z                          |
// |RSL    | RSE=1 : Reset = RSL                           |
// +-------+-----------------------------------------------+
// |TPM    | Trigger Pulse Mode                            |
// |       | 0=Single Trigger(write access to clear)       |
// |       | 1=Pulse Trigger                               |
// +-------+-----------------------------------------------+
// |R13    | 1=ROM.A13 enable (128Kbit, or larger)         |
// +-------+-----------------------------------------------+
// |R14    | 1=ROM.A14 enable (256Kbit, or larger)         |
// +-------+-----------------------------------------------+
// |R15    | 1=ROM.A15 enable (512Kbit, or larger)         |
// +-------+-----------------------------------------------+
// |R16    | 1=ROM.A16 enable (1Mbit  , or larger)         |
// +-------+-----------------------------------------------+
// |R17    | 1=ROM.A17 enable (2Mbit  , or larger)         |
// +-------+-----------------------------------------------+
// |R18    | 1=ROM.A18 enable (4Mbit  , or larger)         |
// +-------+-----------------------------------------------+
// |FL     | 0=A18 location is DIP32.31 (EPROM)            |
// |       | 1=A18 location is DIP32.1  (EEPROM/Flash)     |
// +-------+-----------------------------------------------+
//
// Read Register Spec.
// +-------+-----------------------------------------------+
// |Param. | Sprc.                                         |
// +-------+-----------------------------------------------+
// |DLL    | DLL Lock Signal , 1=lock                      |
// +-------+-----------------------------------------------+
// |PLL    | PLL Lock Signal , 1=lock                      |
// +-------+-----------------------------------------------+
// |VCC    | Target VCC level , 1=Avtive                   |
// +-------+-----------------------------------------------+
// |TRG    | Trigger Ditect signal                         |
// |       | TRG clear when any write access               |
// +-------+-----------------------------------------------+
// |MV,RC  | Design Version/Release code                   |
// +-------+-----------------------------------------------+
//

module romemuf(
  PROBE,EXT,
  RCLK,RA,RD,RADSC_N,RGW_N,RCLK,RADV_N,RCE_N,ROE_N,
//  CN2_38,CN2_40,
  VCLK,VTUNE,INT_CLK);

inout [31:0] PROBE;
output [3:0] EXT;

inout RCLK;
output [16:0] RA;
inout  [31:0] RD;
output RADSC_N;
output ROE_N;
output RGW_N;
output RADV_N; // ADV
output RCE_N;

//input  CN2_38; // GCK1
//input  CN2_40; // GCK0
input  VCLK;
output VTUNE;
input  INT_CLK; // 24MHz

/////////////////////////////////////////////////////////////////////////////
// JTAG MEM I/F
/////////////////////////////////////////////////////////////////////////////
wire [30:0] JA;
wire [7:0] JDI;
wire [7:0] JDO;
wire JDE;
wire JCS,JOE,JWE;

jtagmem jtagmem(
  .JA(JA),
  .JD_I(JDI),
  .JD_O(JDO),
  .JD_E(JDE),
  .JCS(JCS),
  .JOE(JOE),
  .JWE(JWE)
);

/////////////////////////////////////////////////////////////////////////////
// PLL module I/F
/////////////////////////////////////////////////////////////////////////////
wire x_clk,v_clk;

IBUFG int_clk(.I(INT_CLK),.O(x_clk));
IBUFG vco_clk(.I(VCLK),.O(v_clk));

wire [7:0] div_a = (24*2)-1; // 500KHz
reg [7:0] div_b  = 50-1;    // ==Mhz (because DLL2x) 
wire pll_a , pll_b , vco_reset_n;

wire pll_lock;
wire x16k,v16k;

pll pll(
  .clk_a(x_clk),.div_a(div_a),.clkd_a(),.phase_a(pll_a),
  .clk_b(v_clk),.div_b(div_b),.clkd_b(),.phase_b(pll_b),
  .lock(pll_lock)
);

// DLL
wire dll_2x , dll_clk , dll_lock , dll_reset;
BUFG DLL_BUFG(.I(dll_2x),.O(dll_clk));
assign dll_reset = ~pll_lock;

CLKDLL VCLK_DLL(
  .CLKIN(v_clk),
  .CLKFB(dll_clk),
  .CLK2X(dll_2x),
  .LOCKED(dll_lock),
  .RST(dll_reset)
);

/////////////////////////////////////////////////////////////////////////////
// VCO
/////////////////////////////////////////////////////////////////////////////
wire pll_off = ~pll_a & ~pll_b & vco_reset_n; // GATE
wire pll_level = pll_a & vco_reset_n;         // LEVEL
assign vco_reset_n = 1;

// VCO TUNE
OBUFT_S_8 vtune_o(.O(VTUNE),.I(pll_level),.T(pll_off));


/////////////////////////////////////////////////////////////////////////////
// HOST I/F
/////////////////////////////////////////////////////////////////////////////

// ROM
reg  [7:0] rom_do;		// ROM output from PBSRAM

// Trigger 
reg [18:0] TA;

// PBAM
wire ram_clk;
reg [16:0] ram_adr_l;
reg ram_gw_l;
reg ram_oe_l;
reg ram_den;
//reg ram_adsc_l;
reg [31:0] ram_do;
reg [1:0] dsel;

// Access Trigger
reg trg_ll,trg_lh,trg_f1,trg_out;
// Reset Output
reg rst_oe,rst_out;
// Trigger
reg trg_pulse;

reg [7:0]  host_dout;
reg f18en,r18en,r17en,r16en,r15en,r14en,r13en;
reg host_wreq; // Host WriteRequest
reg emu_run;   // Emuration run

// DataLatch & Write Request Flag 
reg wreq_l;

always @(posedge JWE or posedge ram_oe_l)
begin
  if(ram_oe_l)
  begin
    // clear write request after HOST write phase 1
    host_wreq <= 0;
  end else begin
    if(~JA[30])
    begin
      case( JA[1:0] )
      2'b00:ram_do[ 7: 0] <= JDO; 		// increment counter
      2'b01:ram_do[15: 8] <= JDO;
      2'b10:ram_do[23:16] <= JDO;
      2'b11:begin ram_do[31:24] <= JDO; host_wreq <= 1; end
      endcase
    end else begin
      case(JA[3:0])
      1:begin // TA L
        TA[7:0] <= JDO;
        end
      2:begin // TA M
        TA[15:8] <= JDO;
        end
      3:begin // TA H
        TA[18:16] <= JDO[2:0];
        end
      5:begin // PLL clock
	     div_b <= JDO;
        end
      6:begin // WR[6] mode,reset controll
        trg_pulse        <= JDO[4]; // TriggerMode
        rst_out          <= JDO[5]; // Reset Level
        rst_oe           <= JDO[6]; // Reset Enable
        emu_run          <= JDO[7]; // -Emulation / Host read
        end
      7:begin // WR[7] rom selector
        {f18en,r18en,r17en,r16en,r15en,r14en,r13en} <= JDO[6:0];
        end
      endcase
    end
  end
end

/////////////////////////////////////////////////////////////////////////////
// TARGET PROBE I/F
/////////////////////////////////////////////////////////////////////////////
wire [18:0] rom_a;
wire [7:0]  rom_d;
wire rom_vcc;
wire rom_gnd;
wire rom_oe;
wire rom_p1;

wire [18:0] rom_addr;
wire rom_vccs , rom_oes;

OBUFT_S_4  p_o0(.O(PROBE[1]),.I(rom_do[3]),.T(~rom_oes));
IBUF       p_01(.I(PROBE[0]),.O(rom_gnd));
OBUFT_S_4  p_o2(.O(PROBE[3]),.I(rom_do[4]),.T(~rom_oes));
OBUFT_S_4  p_o3(.O(PROBE[2]),.I(rom_do[2]),.T(~rom_oes));
OBUFT_S_4  p_o4(.O(PROBE[5]),.I(rom_do[5]),.T(~rom_oes));
OBUFT_S_4  p_o5(.O(PROBE[4]),.I(rom_do[1]),.T(~rom_oes));
OBUFT_S_4  p_o6(.O(PROBE[7]),.I(rom_do[6]),.T(~rom_oes));
OBUFT_S_4  p_o7(.O(PROBE[6]),.I(rom_do[0]),.T(~rom_oes));
OBUFT_S_4  p_o8(.O(PROBE[9]),.I(rom_do[7]),.T(~rom_oes));
IBUF       p_09(.I(PROBE[8]),.O(rom_a[0]));
IBUF       p_10(.I(PROBE[11]),.O(rom_ce));
IBUF       p_11(.I(PROBE[10]),.O(rom_a[1]));
IBUF       p_12(.I(PROBE[13]),.O(rom_a[10]));
IBUF       p_13(.I(PROBE[12]),.O(rom_a[2]));
IBUF       p_14(.I(PROBE[15]),.O(rom_oe));
IBUF       p_15(.I(PROBE[14]),.O(rom_a[3 ]));
IBUF       p_16(.I(PROBE[17]),.O(rom_a[11]));
IBUF       p_17(.I(PROBE[16]),.O(rom_a[4 ]));
IBUF       p_18(.I(PROBE[19]),.O(rom_a[9 ]));
IBUF       p_19(.I(PROBE[18]),.O(rom_a[5 ]));
IBUF       p_20(.I(PROBE[21]),.O(rom_a[8 ]));
IBUF       p_21(.I(PROBE[20]),.O(rom_a[6 ]));
IBUF       p_22(.I(PROBE[23]),.O(rom_a[13]));
IBUF       p_23(.I(PROBE[22]),.O(rom_a[7 ]));
IBUF       p_24(.I(PROBE[25]),.O(rom_a[14]));
IBUF       p_25(.I(PROBE[24]),.O(rom_a[12]));
IBUF       p_26(.I(PROBE[27]),.O(rom_a[17]));
IBUF       p_27(.I(PROBE[26]),.O(rom_a[15]));
IBUF       p_28(.I(PROBE[29]),.O(rom_a[18]));
IBUF       p_29(.I(PROBE[28]),.O(rom_a[16]));
IBUF       p_30(.I(PROBE[31]),.O(rom_vcc));
IBUF       p_31(.I(PROBE[30]),.O(rom_p1));

// -------------------- HOST Read I/F ---------------------
assign JDI = ~JA[30]? rom_do :           // ReadBack emulation memory
// readback Write register
             JA[3:0]==1 ? TA[7:0] :
             JA[3:0]==2 ? TA[15:8] :
             JA[3:0]==3 ? {5'b0000,TA[18:16]} :
             JA[3:0]==5 ? div_b :
             JA[3:0]==6 ? {emu_run,rst_oe,rst_out,trg_pulse,4'b0000} :
             JA[3:0]==7 ? {1'b0,f18en,r18en,r17en,r16en,r15en,r14en,r13en} :
//Read register
             JA[3:0]==12? 8'h52 : // +0d:'R'
             JA[3:0]==13? 8'h45 : // +0e:'E'
             JA[3:0]==14? 8'h46 : // +0e:'F'
             JA[3:0]==15? 8'h10 : // +0f:Version 01.00
             JA[3:0]==4 ? {4'b0000,dll_lock,pll_lock,rom_vccs,trg_out} : // +04 trg
             8'h00;

// --------------------- PROM pin selector --------------------
wire [31:0] ram_di;

assign rom_addr[12:0] = rom_a[12:0];
assign rom_addr[13] = r13en& rom_a[13];
assign rom_addr[14] = r14en& rom_a[14];
assign rom_addr[15] = r15en& rom_a[15];
assign rom_addr[16] = r16en& rom_a[16];
assign rom_addr[17] = r17en& rom_a[17];
assign rom_addr[18] = r18en& (f18en ? rom_p1 : rom_a[18]);

// DIP28 / DIP32 VCC select
assign rom_vccs       = r16en ? rom_vcc   : rom_a[17];

// data output enable , CS , OE , POWER
assign rom_act = ~rom_gnd && rom_vccs && emu_run;
assign rom_oes = ~rom_ce && ~rom_oe && rom_act;

//!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
// noise filter
reg [18:0] rom_addr_l,rom_addr_l1;
always @(posedge ram_clk)
begin
  rom_addr_l1 <= rom_addr;
  if(rom_addr_l1==rom_addr)
    rom_addr_l <= rom_addr_l1;
end

// -------------------- PBSRAM interface --------------------
// access timming
always @(posedge ram_clk)
begin

  // Write Cycle Controll
  if(wreq_l)
  begin
    ram_oe_l   <= 1'b1;      // HOST-WR:1 = negate OE 
    ram_den    <= ram_oe_l;  // HOST-WR:2 = enable data output
    ram_gw_l   <= ~ram_den;  // HOST-WR:3 = assert GW
    wreq_l     <= ~ram_den;  // (clear host_wreq)
//    ram_adsc_l <= ram_den;
  end else begin
    wreq_l <= host_wreq;     // enter write cycle for next clock
//    ram_adsc_l <= 0;
    ram_den    <= 1'b0;      // HOST-WR:4 data output disable
    ram_gw_l   <= 1'b1;      // HOST-WR:4 negate GW & data bus floating
    ram_oe_l   <= ram_den;   // HOST-WR:5 assert OE
  end

  // PBSRAM data latch (for read cycle)
  if(~ram_oe_l)
    rom_do <= (dsel == 2'b00) ? ram_di[7:0]   :
              (dsel == 2'b01) ? ram_di[15:8]  :
              (dsel == 2'b10) ? ram_di[23:16] :
                                ram_di[31:24];

  // PBRAM Address Selector (Target / Host)
  dsel      <= emu_run ? rom_addr_l[1:0]  : JA[1:0]; 
  ram_adr_l <= emu_run ? rom_addr_l[18:2] : JA[18:2]; 

  // Access Trigger
//  trg_lh <= TA[18:13]==rom_addr[18:13];
//  trg_ll <= (TA[12:0]==rom_addr[12:0]) & rom_oes;
  trg_lh <= TA[18:13]==ram_adr_l[16:11];
  trg_ll <= (TA[12:0]=={ram_adr_l[10:0],dsel}) & rom_oes;
  // 2clock filter
  trg_f1 <= trg_lh & trg_ll;
  // sense
  if(trg_f1 & trg_lh & trg_ll)
    trg_out <= 1;
  else if(JWE || trg_pulse)
    trg_out <= 0; // clear from HOST write
end

/////////////////////////////////////////////////////////////////////////////
// PBSRAM I/O
/////////////////////////////////////////////////////////////////////////////

wire sys_clk = dll_clk & dll_lock & pll_lock;
IOBUF_F_12 rclk_o(.I(sys_clk),.T(1'b0),.O(ram_clk),.IO(RCLK));

OBUF_F_12 radsc_o(.I(1'b0),.O(RADSC_N)); // ram_adsc_l
OBUF_F_12 adc_o(  .I(1'b0),.O(RADV_N));  // 1'b1
OBUF_F_12 rce_o(.I(1'b0),.O(RCE_N));
OBUF_F_12 roe_o(.I(ram_oe_l),.O(ROE_N));
OBUF_F_12 rgw_o(.I(ram_gw_l),.O(RGW_N));
// Address Bus
//assign RA = ram_adr_l;
OBUF_F_12 roe_o0 (.I(ram_adr_l[ 0]),.O(RA[ 0]));
OBUF_F_12 roe_o1 (.I(ram_adr_l[ 1]),.O(RA[ 1]));
OBUF_F_12 roe_o2 (.I(ram_adr_l[ 2]),.O(RA[ 2]));
OBUF_F_12 roe_o3 (.I(ram_adr_l[ 3]),.O(RA[ 3]));
OBUF_F_12 roe_o4 (.I(ram_adr_l[ 4]),.O(RA[ 4]));
OBUF_F_12 roe_o5 (.I(ram_adr_l[ 5]),.O(RA[ 5]));
OBUF_F_12 roe_o6 (.I(ram_adr_l[ 6]),.O(RA[ 6]));
OBUF_F_12 roe_o7 (.I(ram_adr_l[ 7]),.O(RA[ 7]));
OBUF_F_12 roe_o8 (.I(ram_adr_l[ 8]),.O(RA[ 8]));
OBUF_F_12 roe_o9 (.I(ram_adr_l[ 9]),.O(RA[ 9]));
OBUF_F_12 roe_o10(.I(ram_adr_l[10]),.O(RA[10]));
OBUF_F_12 roe_o11(.I(ram_adr_l[11]),.O(RA[11]));
OBUF_F_12 roe_o12(.I(ram_adr_l[12]),.O(RA[12]));
OBUF_F_12 roe_o13(.I(ram_adr_l[13]),.O(RA[13]));
OBUF_F_12 roe_o14(.I(ram_adr_l[14]),.O(RA[14]));
OBUF_F_12 roe_o15(.I(ram_adr_l[15]),.O(RA[15]));
OBUF_F_12 roe_o16(.I(ram_adr_l[16]),.O(RA[16]));
	   
// DataBus
wire ram_dt = ~ram_den;
IOBUF_F_8  rd0 (.I(ram_do[0 ]),.T(ram_dt),.O(ram_di[0 ]),.IO(RD[0 ]));
IOBUF_F_8  rd1 (.I(ram_do[1 ]),.T(ram_dt),.O(ram_di[1 ]),.IO(RD[1 ]));
IOBUF_F_8  rd2 (.I(ram_do[2 ]),.T(ram_dt),.O(ram_di[2 ]),.IO(RD[2 ]));
IOBUF_F_8  rd3 (.I(ram_do[3 ]),.T(ram_dt),.O(ram_di[3 ]),.IO(RD[3 ]));
IOBUF_F_8  rd4 (.I(ram_do[4 ]),.T(ram_dt),.O(ram_di[4 ]),.IO(RD[4 ]));
IOBUF_F_8  rd5 (.I(ram_do[5 ]),.T(ram_dt),.O(ram_di[5 ]),.IO(RD[5 ]));
IOBUF_F_8  rd6 (.I(ram_do[6 ]),.T(ram_dt),.O(ram_di[6 ]),.IO(RD[6 ]));
IOBUF_F_8  rd7 (.I(ram_do[7 ]),.T(ram_dt),.O(ram_di[7 ]),.IO(RD[7 ]));
IOBUF_F_8  rd8 (.I(ram_do[8 ]),.T(ram_dt),.O(ram_di[8 ]),.IO(RD[8 ]));
IOBUF_F_8  rd9 (.I(ram_do[9 ]),.T(ram_dt),.O(ram_di[9 ]),.IO(RD[9 ]));
IOBUF_F_8  rd10(.I(ram_do[10]),.T(ram_dt),.O(ram_di[10]),.IO(RD[10]));
IOBUF_F_8  rd11(.I(ram_do[11]),.T(ram_dt),.O(ram_di[11]),.IO(RD[11]));
IOBUF_F_8  rd12(.I(ram_do[12]),.T(ram_dt),.O(ram_di[12]),.IO(RD[12]));
IOBUF_F_8  rd13(.I(ram_do[13]),.T(ram_dt),.O(ram_di[13]),.IO(RD[13]));
IOBUF_F_8  rd14(.I(ram_do[14]),.T(ram_dt),.O(ram_di[14]),.IO(RD[14]));
IOBUF_F_8  rd15(.I(ram_do[15]),.T(ram_dt),.O(ram_di[15]),.IO(RD[15]));
IOBUF_F_8  rd16(.I(ram_do[16]),.T(ram_dt),.O(ram_di[16]),.IO(RD[16]));
IOBUF_F_8  rd17(.I(ram_do[17]),.T(ram_dt),.O(ram_di[17]),.IO(RD[17]));
IOBUF_F_8  rd18(.I(ram_do[18]),.T(ram_dt),.O(ram_di[18]),.IO(RD[18]));
IOBUF_F_8  rd19(.I(ram_do[19]),.T(ram_dt),.O(ram_di[19]),.IO(RD[19]));
IOBUF_F_8  rd20(.I(ram_do[20]),.T(ram_dt),.O(ram_di[20]),.IO(RD[20]));
IOBUF_F_8  rd21(.I(ram_do[21]),.T(ram_dt),.O(ram_di[21]),.IO(RD[21]));
IOBUF_F_8  rd22(.I(ram_do[22]),.T(ram_dt),.O(ram_di[22]),.IO(RD[22]));
IOBUF_F_8  rd23(.I(ram_do[23]),.T(ram_dt),.O(ram_di[23]),.IO(RD[23]));
IOBUF_F_8  rd24(.I(ram_do[24]),.T(ram_dt),.O(ram_di[24]),.IO(RD[24]));
IOBUF_F_8  rd25(.I(ram_do[25]),.T(ram_dt),.O(ram_di[25]),.IO(RD[25]));
IOBUF_F_8  rd26(.I(ram_do[26]),.T(ram_dt),.O(ram_di[26]),.IO(RD[26]));
IOBUF_F_8  rd27(.I(ram_do[27]),.T(ram_dt),.O(ram_di[27]),.IO(RD[27]));
IOBUF_F_8  rd28(.I(ram_do[28]),.T(ram_dt),.O(ram_di[28]),.IO(RD[28]));
IOBUF_F_8  rd29(.I(ram_do[29]),.T(ram_dt),.O(ram_di[29]),.IO(RD[29]));
IOBUF_F_8  rd30(.I(ram_do[30]),.T(ram_dt),.O(ram_di[30]),.IO(RD[30]));
IOBUF_F_8  rd31(.I(ram_do[31]),.T(ram_dt),.O(ram_di[31]),.IO(RD[31]));


// External Signal Output
assign EXT[0] = ~rom_act;    // Address Enable
assign EXT[1] = ~rom_oes;	  // Data Enable
assign EXT[2] = trg_out;                 // AccessTrigger
assign EXT[3] = ~rst_oe  ? 1'bz : rst_out; // Reset Output

endmodule
